/* $Id: ike_spmif.c,v 1.8 2005/10/06 14:54:23 inoue Exp $ */

/*
 * Copyright (C) 2004 WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * IKE-SPMIF interface bridge
 */

#include <config.h>

#include <stdio.h>
#include <string.h>
#ifdef HAVE_INTTYPES_H
#  include <inttypes.h>
#endif
#include <sys/types.h>
#if TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#  if HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif

#include "racoon.h"
#include "isakmp_impl.h"
#include "debug.h"

#ifndef PRIu64
/* hope this works */
#  define PRIu64	"lld"
#endif

int debug_spmif;

static int spmif_socket = -1;

static int ike_spmif_post_slid_callback();

int
ike_spmif_init(void)
{
	spmif_socket = spmif_init();
	return spmif_socket;
}

int
ike_spmif_socket(void)
{
	return spmif_socket;
}

int
ike_spmif_poll(void)
{
	if (spmif_handler(spmif_socket) != 0) {
		/* no reason to work anymore */
		spmif_clean(spmif_socket);
		spmif_socket = -1;
		isakmp_log(0, 0, 0, 0,
			   PLOG_INTERR, PLOGLOC,
			   "spmd I/F broken: This is fatal and iked needs to be restarted\n");
		return -1;
	}
	return 0;
}

int
ike_spmif_post_slid(void *tag, uint32_t spid)
{
	return spmif_post_slid(spmif_socket, &ike_spmif_post_slid_callback,
			       tag, spid);
}

static int
ike_spmif_post_slid_callback(void *tag, char *slid)
{
	int err;

	err = isakmp_initiate_cont(tag, slid);
	if (err) {
		struct isakmp_acquire_request *req;
		struct rcpfk_msg param;

		req = (struct isakmp_acquire_request *)tag;
		param.seq = req->request_msg_seq;
		param.eno = err;
		/* rcpfk_send_acquire() requires satype eventhough kernel doesn't use it */
		param.satype = RCT_SATYPE_ESP;	/* XXX */
		req->callback_method->acquire_error(&param);
	}

	return 0;		/* return value ignored by caller */
}
